; MessageBox
; A non-interactive notification window.
; Inputs: hl -> MessageBox Structure
; Outputs: hl -> Byte after MessageBox structure
;
; MessageBox Structure:
; .db "Title",0            ; The title (inverse text)
; .db num_lines            ; The number of lines of text to be displayed (not including the title)
; .db "Line 1",0           ; The text to display
; .db "Line 2",0           ; ...
; .db "Line 3",0           ; ...
; .db "Line 4",0           ; ...
;
;
; Comments:
;  Num_lines can be zero.
;  No matter what num_lines is, the rectangle will be 4 lines high, and about 12 capital letters wide.
;  Everything is displayed using the small font.
;  This routine resets textwrite automatically.
;  The title is centered, and the text is left aligned.
;  Nothing is saved to the graph buffer.
;

guiMessageBox:
 res textwrite,(iy+sgrflags)
 set plotLoc,(iy+plotFlags)
 res fracDrawLFont,(iy+fontFlags)
 push hl
 ld hl,14*256+22
 ld de,46*256+73
 B_CALL _DrawRectBorderC
 ld hl,15*256+23
 ld de,20*256+72
 B_CALL _FillRect
 pop hl
 set textinverse,(iy+textflags)
 ld de,14*256+47
 ld (pencol),de
 call centertext
 res textinverse,(iy+textflags)
 ld de,21*256+24
 call multilinetextL
 ret


; MessageBoxEX
; A semi-interactive notification window with extra options.
; Inputs: hl -> MessageBoxEX Structure
; Outputs: hl -> Byte after MessageBoxEX structure, except hl -> animateroutine if it exists
;          a = keycode if MB_Waitkey was used.
;
; MessageBoxEX Structure:
; .db flags                            ; see below
; .db height,width,titleheight         ; height of entire box, width of entire box, height of dark region
; .db num_titles                       ; number of lines of text in the title
; .db "Title Line 1",0                 ; the actual lines of text for the title
; .db "Title Line 2",0                 ; ...
; .db num_lines                        ; number of lines of text in the box (not including title)
; .db "Line 1",0                       ; the actual lines of text
; .db "Line 2",0                       ; ...
; .db "Line 3",0                       ; ...
; .db "Line 4",0                       ; ...
; [.db delaytime]                      ; the length of the delay (if used)
; [.dw animateroutine]                 ; a pointer to a routine in ram to be called during delay/waitkey
;
; flags:                         SET (nz)                               RESET (z)
;  Bit 0: MB_Buffer              Display to buffer and screen           Display to screen (not buffer)
;  Bit 1: MB_CenterTitle         Center title                           Left align title
;  Bit 2: MB_CenterText          Center text                            Left align text
;  Bit 3: MB_BigTitle            Title in large font                    Title in small font
;  Bit 4: MB_BigText             Text in large font                     Text in small font
;  Bit 5: MB_Delay               Execute delaytime halts                Just return without a delay
;  Bit 6: MB_Waitkey             Wait until 2nd or Enter pressed        Just return without keypress
;  Bit 7: MB_Animate             Run a callback during delay/waitkey    No callback
;
; Comments:
;  The box will be centered on the screen.
;  If both delay and waitkey are used, the delay will come first.
;  The callback will be run repeatedly with one halt and a couple of lines of code in between calls.
;  Do not use any MirageOS calls during the callback routine. The flash page is NOT reset.
;  Delaytime and animateroutine should be there ONLY if their respective flags are set. If you are using
;   an animation routine but no delay, do not include a dummy value for delaytime.
;  If MB_Buffer is set, a grbufcpy will be executed.
;  The standard MessageBox has values 2,33,52,7,1 for the first 5 bytes.
;

MB_Buffer         equ   0
MB_CenterTitle    equ   1
MB_CenterText     equ   2
MB_BigTitle       equ   3
MB_BigText        equ   4
MB_Delay          equ   5
MB_Waitkey        equ   6
MB_Animate        equ   7

guiMessageBoxEX:
 push hl
 push hl
 pop ix
 set plotLoc,(iy+plotFlags)
 res textwrite,(iy+sgrflags)
 bit MB_Buffer,(ix)
 jr z,guiMessageBoxEXNoBuffer
 res plotLoc,(iy+plotFlags)
 set textwrite,(iy+sgrflags)
guiMessageBoxEXNoBuffer:
 ld a,(ix+1)      ;height
 sra a
 neg
 add a,31
 ld h,a
 add a,(ix+1)
 ld d,a
 ld a,(ix+2)      ;width
 sra a
 neg
 add a,47
 ld l,a
 add a,(ix+2)
 ld e,a
 push hl
 push de
 push ix
 B_CALL _DrawRectBorderC
 pop ix
 pop de
 pop hl
 ld a,(ix+3)
 add a,h
 ld d,a
 B_CALL _FillRect
 pop hl           ;get pointer again
 ld de,4
 add hl,de
 set textinverse,(iy+textflags)
 res fracDrawLFont,(iy+fontFlags)
 bit MB_BigTitle,(ix)
 jr z,guiMessageBoxEXTitleSmall
 set fracDrawLFont,(iy+fontFlags)
guiMessageBoxEXTitleSmall:
 bit MB_CenterTitle,(ix)
 jr z,guiMessageBoxEXTitleLeft
 ld a,(ix+1)
 sra a
 neg
 add a,31
 bit fracDrawLFont,(iy+fontFlags)
 jr z,guiMessageBoxEXTitleNoShift
 inc a
guiMessageBoxEXTitleNoShift:
 ld d,a
 ld e,47
 ld (pencol),de
 call multilinetextC
 jr guiMessageBoxEXTitleDone
guiMessageBoxEXTitleLeft:
 ld a,(ix+1)
 sra a
 neg
 add a,31
 bit fracDrawLFont,(iy+fontFlags)
 jr z,guiMessageBoxEXTitleNoShift2
 inc a
guiMessageBoxEXTitleNoShift2:
 ld d,a
 ld a,(ix+2)
 sra a
 neg
 add a,48
 ld e,a
 ld (pencol),de
 call multilinetextL
guiMessageBoxEXTitleDone:
 res textinverse,(iy+textflags)
 res fracDrawLFont,(iy+fontFlags)
 bit MB_BigText,(ix)
 jr z,guiMessageBoxEXTextSmall
 set fracDrawLFont,(iy+fontFlags)
guiMessageBoxEXTextSmall:
 bit MB_CenterText,(ix)
 jr z,guiMessageBoxEXTextLeft
 ld a,(ix+1)
 sra a
 neg
 add a,32
 bit fracDrawLFont,(iy+fontFlags)
 jr z,guiMessageBoxEXTextNoShift
 inc a
guiMessageBoxEXTextNoShift:
 add a,(ix+3)
 ld d,a
 ld e,47
 call multilinetextC
 jr guiMessageBoxEXTextDone
guiMessageBoxEXTextLeft:
 ld a,(ix+1)
 sra a
 neg
 add a,32
 bit fracDrawLFont,(iy+fontFlags)
 jr z,guiMessageBoxEXTextNoShift2
 inc a
guiMessageBoxEXTextNoShift2:
 add a,(ix+3)
 ld d,a
 ld a,(ix+2)
 sra a
 neg
 add a,49
 ld e,a
 call multilinetextL
guiMessageBoxEXTextDone:
 bit MB_Buffer,(ix)
 jr z,guiMessageBoxEXNoBuffer2
 B_CALL _grbufcpy
guiMessageBoxEXNoBuffer2:
 bit MB_Delay,(ix)
 jr z,guiMessageBoxEXNoDelay
 ld b,(hl)
 inc hl
guiMessageBoxEXDelayLoop:
 ei
 halt
 bit MB_Animate,(ix)
 jr z,guiMessageBoxEXDelayNoAnimate
 push hl
 ld de,guiMessageBoxEXDelayReturn
 push de
 ld a,(hl)
 inc hl
 ld h,(hl)
 ld l,a
 jp (hl)    ;really just a call
guiMessageBoxEXDelayReturn:
 pop hl
guiMessageBoxEXDelayNoAnimate:
 djnz guiMessageBoxEXDelayLoop
guiMessageBoxEXNoDelay:
 ;reset some flags here...
 res plotLoc,(iy+plotflags)
 res textwrite,(iy+sgrflags)
 res fracDrawLFont,(iy+fontFlags)
 bit MB_Waitkey,(ix)
 ret z
guiMessageBoxEXWaitkeyLoop:
 ei
 halt
 B_CALL _getcsc
 cp sk2nd
 ret z
 cp skEnter
 ret z
 bit MB_Animate,(ix)
 jr z,guiMessageBoxEXWaitkeyNoAnimate
 push hl
 ld de,guiMessageBoxEXWaitkeyReturn
 push de
 ld a,(hl)
 inc hl
 ld h,(hl)
 ld l,a
 jp (hl)    ;really just a call
guiMessageBoxEXWaitkeyReturn:
 pop hl
guiMessageBoxEXWaitkeyNoAnimate:
 jr guiMessageBoxEXWaitkeyLoop
